/*
 * edge_detection.cpp
 *
 *  Created on: Aug 11, 2017
 *      Author: julien
 */

#include "edge_detection.hpp"

EdgeDetection::EdgeDetection(int height, int width, int translation_x, int translation_y)
	: r(translation_x, translation_y, width-2*translation_x, height-translation_y),
	  lower_yellow(10,0,100), upper_yellow(40, 255, 255),
	  lower_white(0, 200, 0), upper_white(255, 255, 255),
	  trapo_mask(height-translation_y, width-2*translation_x, CV_8UC1, Scalar(0))
{
	// create a trapezoid mask based upon the points inserted into co_ordinates
	vector< vector<Point> >  co_ordinates;
	// create a new vector inside the other vector to be conform with drawContours
	co_ordinates.push_back(vector<Point>());
	// put all the corner points of the polygon (trapezoid) into the first vector of co_ordinates
	co_ordinates[0].push_back(Point(r.width/3, 0)); // upper left point
	co_ordinates[0].push_back(Point(r.width*2/3, 0)); // upper right point
	co_ordinates[0].push_back(Point(r.width, r.height)); // lower right point
	co_ordinates[0].push_back(Point(0, r.height)); // lower left point
	// draw the shape to serve as mask later
	drawContours( trapo_mask,co_ordinates,0, Scalar(255),CV_FILLED, 8 );
}

void EdgeDetection::detect(Mat* src_processed, Mat* grad_x, Mat* grad_y){

	//create region of interest
	roi = (*src_processed)(r);

	// convert roi to hls
	cvtColor(roi, roi_hls, COLOR_BGR2HLS);
	// convert roi to grayscale
	cvtColor(roi, roi_grey, COLOR_BGR2GRAY );



	// create mask inside the color threshold for white and yellow
	// (only look at those colors and create a dynamic mat based upon them)
	inRange(roi_hls, lower_yellow, upper_yellow, yellow_mask); //outputs in a one channel CV_8U mat
	inRange(roi_hls, lower_white, upper_white, white_mask);

	// create a combined mask with yellow and white
	bitwise_or(white_mask, yellow_mask, combined_mask);

	// only look at the tropzoid part of the picture
	// this is currently not applied, as the cam sits not far enough back
//	bitwise_and(combined_mask, trapo_mask, combined_mask);

	// invert the mask to filter roi_grey
	bitwise_not(combined_mask, combined_mask);
	roi_grey.setTo(0, combined_mask);

//	imshow("test", roi_grey);

	// save inward gradients as a lookup table for later use in concatenation
	Scharr(roi_grey, *grad_x, ddepth, 1, 0, scale, delta, BORDER_DEFAULT );
	Scharr(roi_grey, *grad_y, ddepth, 0, 1, scale, delta, BORDER_DEFAULT );

}


